﻿namespace KPGBase.Excercises
{
    internal class ComplexPoint
    {
        public double real;
        public double imaginary;

        public ComplexPoint(double real, double imaginary)
        {
            this.real = real;
            this.imaginary = imaginary;
        }

        public void NextIteration(double cReal, double cImag)
        {
            // New iteration of M-set sequence
            // z_n+1 = z_n^2 + c
            // c = cReal + cImag * i

            double newReal = real * real - imaginary * imaginary + cReal; 
            double newImaginary = 2 * real * imaginary + cImag;

            real = newReal;
            imaginary = newImaginary;
        }

        public bool InMandelbrot()
        {
            // Is the complex point in M-set?

            return this.real * this.real + this.imaginary * this.imaginary < 4;
        }
    }

    internal class Excercise09 : BasicExcercise
    {
        /// <summary> Maximum number of iterations </summary>
        private double MAX_ITERATIONS = 50.0;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="canvas"> Image canvas </param>
        public Excercise09(Bitmap canvas) : base(canvas) { }

        /// <summary>
        /// Draw M-set
        /// </summary>
        public void DrawMandel()
        {
            // Area that the contains the M-set
            double xMin = -2.1;
            double xMax = 1;
            double yMin = -1.3;
            double yMax = 1.3;

            // Compute the "step" in x and y directions
            //  - interval (-2.1; 1) has to stretch over the whole width of the window
            //  - interval (-1.3; 1.3) has to stretch over the whole height of the window
            // For each pixel in the canvas
            //  - set counter of iteration to 0
            //  - starting complex point z is (0,0)
            //  - while counter < MAX_ITERATIONS and z is in M-set
            //      - counter++
            //      - compute next iteration of z - location of the point is used as c
            //  - if after MAX_ITERATIONS iterations the point is still in M-set color it white
            //  - otherwise color it black

            ClearImage();

            double intigralX = (xMax - xMin) / canvas.Width;
            double intigralY = (yMax - yMin) / canvas.Height;

            double x = xMin;
            for (int i = 0; i < canvas.Width; i++)
            {
                double y = yMin;
                for (int j = 0; j < canvas.Height; j++)
                {
                    int iteration = 0;
                    ComplexPoint z = new ComplexPoint(0, 0);
                    while (iteration < MAX_ITERATIONS && z.InMandelbrot())
                    {
                        iteration++;
                        z.NextIteration(x, y);
                    }

                    double perc = iteration / MAX_ITERATIONS;
                    int color = ((int)(perc * 255));

                    canvas.SetPixel(i, j, Color.FromArgb(color, color, color));

                    y += intigralY;
                }
                x += intigralX;
            }

        }

        /// <summary>
        /// Draw J-set
        /// </summary>
        public void DrawJulia()
        {
            // Area that the contains the J-set
            double xMin = -1.5;
            double xMax = 1.5;
            double yMin = -1;
            double yMax = 1;

            // Complex constant c >> used for the computation of the next iteration of z: z_n+1 = z + c;
            ComplexPoint c = new ComplexPoint(-0.7, 0.27015);

            // Similar computation as in M-set
            // Except:
            //  - starting z corresponds to the point location
            //  - complex point c is always used to compute the next iteration of z

            ClearImage();

            double intigralX = (xMax - xMin) / canvas.Width;
            double intigralY = (yMax - yMin) / canvas.Height;

            double x = xMin;
            for (int i = 0; i < canvas.Width; i++)
            {
                double y = yMin;
                for (int j = 0; j < canvas.Height; j++)
                {
                    int iteration = 0;
                    ComplexPoint z = new ComplexPoint(x, y);
                    while (iteration < MAX_ITERATIONS && z.InMandelbrot())
                    {
                        z.NextIteration(c.real, c.imaginary);
                        iteration++;
                    }

                    double perc = iteration / MAX_ITERATIONS;
                    int color = ((int)(perc * 255));
                    canvas.SetPixel(i, j, Color.FromArgb(color, color, color));

                    y += intigralY;
                }
                x += intigralX;
            }
        }


        /// <summary>
        /// Set the maximum number of iterations
        /// </summary>
        internal double MaxIterations
        {
            get { return this.MAX_ITERATIONS; }
            set { this.MAX_ITERATIONS = value; }
        }

    }
}
